// aio1_out_top : Handles the D2A conversion and parallel to serial conversion
// Also takes care of required clock and signalling for the AD7303 (D2A chip) 

module aio1_out_d2a(
         clk_20,       // 50 MHz (20 ns) clock
         reset,        // Push button reset
         fir_data_out, // FIR data from the filter
         sync_bar,     // control signal for the AD7303 ( D2A )
         DAC_in,       // Serial data input to the D2C
         DACLK         // Clock to the D2A
         );
   
   input clk_20,reset;
   input [7:0]fir_data_out;
   output sync_bar,DAC_in,DACLK;
   
   wire clk_80,DACLK;
   reg  [1:0]count_clk;
   reg  [5:0]count_data;
   reg  DAC_in,sync_bar;
   
   reg [15:0]dac_data;
   
   // Generating clk_80 - This is the D2A Clock ( DACLK ) signal
   
   always @(posedge clk_20 or posedge reset) begin
       if(reset == 1'b1)begin
           count_clk <= 0;
       end
       else begin
           count_clk <= count_clk + 1;
       end
   end

   assign clk_80 = count_clk[1];
   assign DACLK = ~clk_80;
   
   // Parallel to serial conversion of fir_data_out
   // The serial signal is input to the D2A
   
   always @(posedge clk_80 or posedge reset)begin
       if(reset == 1)begin
           count_data <= 0;
           sync_bar <= 1'b1;
           DAC_in <= 1'b0;
       end
       else begin
           if(count_data <= 23)begin
               sync_bar <= 1'b1;
               DAC_in <= 1'b0;
           end
           else if(count_data >= 40 && count_data <= 63)begin
               sync_bar <= 1'b1;
               DAC_in <= 1'b0;
           end
           else begin
               sync_bar <= 1'b0;
               case(count_data)
                   
                   // Control bits
                   
                   6'd24: DAC_in <= 1'b0;   // ~INT/EXT
                   6'd25: DAC_in <= 1'b0;   // X
                   6'd26: DAC_in <= 1'b1;   // LDAC
                   6'd27: DAC_in <= 1'b0;   // PDB
                   6'd28: DAC_in <= 1'b0;   // PDA
                   6'd29: DAC_in <= 1'b0;   // ~A/B
                   6'd30: DAC_in <= 1'b0;   // CR1
                   6'd31: DAC_in <= 1'b0;   // CR0
                   
                   // Data bits
                   
                   6'd32: DAC_in <= fir_data_out[7];   // D7
                   6'd33: DAC_in <= fir_data_out[6];   // D6
                   6'd34: DAC_in <= fir_data_out[5];   // D5
                   6'd35: DAC_in <= fir_data_out[4];   // D4
                   6'd36: DAC_in <= fir_data_out[3];   // D3
                   6'd37: DAC_in <= fir_data_out[2];   // D2
                   6'd38: DAC_in <= fir_data_out[1];   // D1
                   6'd39: DAC_in <= fir_data_out[0];   // D0
                   
                   default: DAC_in <= 1'b0;
              endcase
           end
           
           count_data <= count_data + 1;
       end
   end

endmodule
