// aio1_top: Top module of the FIR filter implementation on AIO1

module aio1_wrapper(
         clk_20,      // 50 MHz (20 ns) clock inoyut from FPGA
         ADOUT,       // This is the serial digital output from the A2D
         reset_in,    // Push Button reset
         conv_start,  // Control signal for the A2D (convert start = ~CONVST on AD7823 chip)
         ADCLK,       // Clock for the A2D
         
         sync_bar,    // Control signal for the D2A
         DAC_in,      // Serial digital input for the D2A
         DACLK,       // Clock for the D2A 
         fir_data_out,
 	 an0,an1,an2,an3,
	 out
         );
            
   input clk_20;
   input ADOUT;
   input reset_in;
   output conv_start;
   output ADCLK;
   output sync_bar;
   output DAC_in;
   output DACLK;
   output [7:0]fir_data_out;
   output [6:0]out;
   output an0,an1,an2,an3;

   wire reset;
   wire [7:0]fir_data_out;

   debounce deb1(reset_in, clk, reset);
   
   // aio1_in_top: Handles the serial digital input from the A2D and also
   // generates required signalling for the AD7823 (A2D chip)
   
   // This module converts serial into parallel data and instantiates 'fir' module
   // and outputs fir_data_out[7:0]
   
   aio1_in_a2d m1(clk_20,
                  ADOUT,
                  reset_in,
                  conv_start,
                  ADCLK,
                  fir_data_out);
   
   // aio1_out_top : Handles the D2A conversion and parallel to serial conversion
   // Also takes care of required clock and signalling for the AD7303 (D2A chip) 
   
   aio1_out_d2a m2(   clk_20,
                      reset_in,
                      fir_data_out,
                      sync_bar,
                      DAC_in,
                      DACLK    );
   
   sevensegment ss1(reset_in,clk_20,out,an0,an1,an2,an3, fir_data_out[3:0],fir_data_out[7:4],4'd0,4'd0);

endmodule
     
