module booth_multiplier(a,b,prod);
    input  [7:0] a,b;
    output [15:0]prod;
    
    reg  [15:0]prod;
    wire [15:0]pp0,pp1,pp2,pp3;
    
    booth_encoder ben0(a,{b[1:0],1'b0},pp0);
    booth_encoder ben1(a,b[3:1],pp1);
    booth_encoder ben2(a,b[5:3],pp2);
    booth_encoder ben3(a,b[7:5],pp3);
    
    always @(pp0 or pp1 or pp2 or pp3)
        prod[15:0] = pp0 + (pp1 << 2) + (pp2 << 4) + (pp3 << 6);
    
endmodule

module booth_encoder(a,b,pp);
    input  [7:0] a;  // 8 bit  multiplicand
    input  [2:0] b;  // 3 bit  multiplier
    output [15:0]pp; //16 bit partial product
    
    reg [15:0]pp;
    reg [7:0] na;
        
    always @(a or b) begin
        na[7:0] = ~a + 1;
        case(b)
            3'b000: pp = 16'd0;
            3'b001: pp = {{8{a[7]}},a};
            3'b010: pp = {{8{a[7]}},a};
            3'b011: pp = {{8{a[7]}},a} << 1;
            3'b100: pp = {{8{na[7]}},na} << 1;
            3'b101: pp = {{8{na[7]}},na};
            3'b110: pp = {{8{na[7]}},na};
            3'b111: pp = 16'd0;
       endcase
   end
endmodule
