
module fir(clk,reset,inp,outp);
    
    parameter N = 17; // Order of the FIR filter
    
    input clk,reset;
    input [7:0]inp;
    output [7:0]outp;
    
    reg  [7:0]x[0:N];
    wire [7:0]coeff[0:N];
    wire [15:0]m[0:N];
    wire [20:0]m_ext[0:N];
    wire [20:0]sum;

    wire [15:0]outp16;
    
    assign coeff[0]  = 8'b11111101;
    assign coeff[1]  = 8'b00000010;
    assign coeff[2]  = 8'b00001000;
    assign coeff[3]  = 8'b00000010;
    assign coeff[4]  = 8'b11110001;
    assign coeff[5]  = 8'b11101001;
    assign coeff[6]  = 8'b00001000;
    assign coeff[7]  = 8'b01001001;
    assign coeff[8]  = 8'b01111111;
    assign coeff[9]  = 8'b01111111;
    assign coeff[10] = 8'b01001001;
    assign coeff[11] = 8'b00001000;
    assign coeff[12] = 8'b11101001;
    assign coeff[13] = 8'b11110001;
    assign coeff[14] = 8'b00000010;
    assign coeff[15] = 8'b00001000;
    assign coeff[16] = 8'b00000010;
    assign coeff[17] = 8'b11111101;

    
    always @(posedge clk) begin
      if(reset == 0)begin
        x[0]  <= inp;
        x[1]  <= x[0];
        x[2]  <= x[1];
        x[3]  <= x[2];
        x[4]  <= x[3];
        x[5]  <= x[4];
        x[6]  <= x[5];
        x[7]  <= x[6];
        x[8]  <= x[7];
        x[9]  <= x[8];
        x[10] <= x[9];
        x[11] <= x[10];
        x[12] <= x[11];
        x[13] <= x[12];
        x[14] <= x[13];
        x[15] <= x[14];
        x[16] <= x[15];
        x[17] <= x[16];
      end
    end

    booth_multiplier bm0  (x[0],  coeff[0],  m[0]);
    booth_multiplier bm1  (x[1],  coeff[1],  m[1]);
    booth_multiplier bm2  (x[2],  coeff[2],  m[2]);
    booth_multiplier bm3  (x[3],  coeff[3],  m[3]);
    booth_multiplier bm4  (x[4],  coeff[4],  m[4]);
    booth_multiplier bm5  (x[5],  coeff[5],  m[5]);
    booth_multiplier bm6  (x[6],  coeff[6],  m[6]);
    booth_multiplier bm7  (x[7],  coeff[7],  m[7]);
    booth_multiplier bm8  (x[8],  coeff[8],  m[8]);
    booth_multiplier bm9  (x[9],  coeff[9],  m[9]);
    booth_multiplier bm10 (x[10], coeff[10], m[10]);
    booth_multiplier bm11 (x[11], coeff[11], m[11]);
    booth_multiplier bm12 (x[12], coeff[12], m[12]);
    booth_multiplier bm13 (x[13], coeff[13], m[13]);
    booth_multiplier bm14 (x[14], coeff[14], m[14]);
    booth_multiplier bm15 (x[15], coeff[15], m[15]);
    booth_multiplier bm16 (x[16], coeff[16], m[16]);
    booth_multiplier bm17 (x[17], coeff[17], m[17]);
    
    // Sign Extend the multiplier outputs
    assign m_ext[0] = { {5{m[0][15]}} , m[0] };
    assign m_ext[1] = { {5{m[1][15]}} , m[1] };
    assign m_ext[2] = { {5{m[2][15]}} , m[2] };
    assign m_ext[3] = { {5{m[3][15]}} , m[3] };
    assign m_ext[4] = { {5{m[4][15]}} , m[4] };
    assign m_ext[5] = { {5{m[5][15]}} , m[5] };
    assign m_ext[6] = { {5{m[6][15]}} , m[6] };
    assign m_ext[7] = { {5{m[7][15]}} , m[7] };
    assign m_ext[8] = { {5{m[8][15]}} , m[8] };
    assign m_ext[9] = { {5{m[9][15]}} , m[9] };
    assign m_ext[10]= { {5{m[10][15]}} , m[10] };
    assign m_ext[11]= { {5{m[11][15]}} , m[11] };
    assign m_ext[12]= { {5{m[12][15]}} , m[12] };
    assign m_ext[13]= { {5{m[13][15]}} , m[13] };
    assign m_ext[14]= { {5{m[14][15]}} , m[14] };
    assign m_ext[15]= { {5{m[15][15]}} , m[15] };
    assign m_ext[16]= { {5{m[16][15]}} , m[16] };
    assign m_ext[17]= { {5{m[17][15]}} , m[17] };

    
    assign sum = m_ext[0] + m_ext[1] + m_ext[2]  + m_ext[3] +  m_ext[4] + m_ext[5] + m_ext[6] + m_ext[7] + m_ext[8] + m_ext[9] + m_ext[10]  +m_ext[11]  +m_ext[12]  +m_ext[13]  +m_ext[14]  +m_ext[15]  +m_ext[16] + m_ext[17];
    
    assign outp = sum[15:8];
 
endmodule
    
